from MRUs.LU import lu_factorization
from MRUs.QR import qr_factorization
from MRUs.Eelementary_Row_Operations import *
from MRUs.Orthogonal_Reduction import orthogonal_reduction
from MRUs.URV import urv_factorization
import numpy as np
np.set_printoptions(precision=3)


import numpy as np
from MRUs.Eelementary_Row_Operations import *


def lu_factorization(A: np.array):
    print('LU分解')
    A = A.copy()
    # M and N are the number of rows and columns of matrix A respectively
    M, N = A.shape
    L_inverse = np.eye(M)
    
    for col in range(N):
        for row in range(col+1, M):
            P = np.eye(M)
            if A[row, col] == 0:
                continue
            elif A[col, col] == 0:
                operator_1(A, col, row)
                operator_1(P, col, row)
                L_inverse = np.dot(P, L_inverse)
                continue
            else:
                w_tmp = -A[row, col] / A[col, col]
                operator_3(A, row, col, w_tmp)
                operator_3(P, row, col, w_tmp)
                L_inverse = np.dot(P, L_inverse)
    
    return np.linalg.inv(L_inverse), A


import numpy as np


def givens(A: np.array):
    print('givens reduction')
    R = A.copy()
    # M and N are the number of rows and columns of matrix R respectively
    M, N = R.shape
    Q = np.eye(M)
    for col in range(min(M, N) - 1):
        for row in range(col + 1, M):
            a = R[col, col]
            b = R[row, col]
            base = np.sqrt(a ** 2 + b ** 2)
            Q_tmp = np.eye(M)
            Q_tmp[row, row] = Q_tmp[col, col] = a / base
            Q_tmp[col, row] = b / base
            Q_tmp[row, col] = -b / base
            R = np.dot(Q_tmp, R)
            Q = np.dot(Q_tmp, Q)
    return Q.T, R


def householder(A: np.array):
    print('householder reduction')
    R = A.copy()
    # M and N are the number of rows and columns of matrix R respectively
    M, N = R.shape
    Q = np.eye(M)
    for col in range(min(M, N) - 1):
        Q_all = np.eye(M)
        R_j = np.array([R[col: M, col]]).T
        e1 = np.zeros_like(R_j)
        e1[[0]] = 1
        R_j = R_j - np.sqrt(np.dot(R_j.T, R_j)) * e1
        Q_tmp = np.eye(M - col) - 2 * np.dot(R_j, R_j.T) / np.dot(R_j.T, R_j)
        Q_all[col: M, col: M] = Q_tmp
        Q = np.dot(Q_all, Q)
        R = np.dot(Q_all, R)
    return Q.T, R


def orthogonal_reduction(A: np.array, core='householder'):
    print('正交分解')
    assert(core in ['householder', 'givens'])
    if core == 'householder':
        return householder(A)
    else:
        return givens(A)


import numpy as np


def qr_factorization(A: np.array):
    print('QR分解')
    A = A.copy()
    # M and N are the number of rows and columns of matrix A respectively
    M, N = A.shape
    Q = np.empty_like(A)
    R = np.zeros((M, M))
    Q_cnt = 0
    for col in range(N):
        ui = A[:, [col]]
        for col_Q in range(0, Q_cnt):
            u_pre = Q[:, [col_Q]]
            inner_product = np.dot(u_pre.T, ui)
            R[col_Q, col] = inner_product
            ui = ui - u_pre*inner_product
        ui_norm = np.sqrt(np.dot(ui.T, ui))
        R[col, col] = ui_norm
        ui = ui / ui_norm
        Q[:, [col]] = ui
        Q_cnt += 1
    return Q, R    
        


from setuptools import setup,find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
setup(
    name = "MRUs",       
    version = "0.0.5",
    author = "MingjunXu",    	
    description = "Matrix Reduction Utils",		
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages = find_packages(), 
    requires=['numpy'],
    exclude = ["*.log",],

)
